@@ -1,14 +1,17 @@
 import logging
 import os
 from glob import glob
-from copy import copy
+from copy import copy, deepcopy
 from sortedcontainers import SortedDict
 
 import kcapi
 
-from kcloader.resource import SingleResource, ResourcePublisher, UpdatePolicy
+from kcloader.resource import SingleResource
+from kcloader.resource.default_client_scope_resource import ClientOptionalClientScopeManager, ClientDefaultClientScopeManager
 from kcloader.resource.role_resource import find_sub_role, BaseRoleManager, BaseRoleResource
 from kcloader.tools import lookup_child_resource, read_from_json, find_in_list, get_path
+from kcloader.resource.scope_mappings import ClientScopeMappingsAllClientsManager, ClientScopeMappingsRealmManager
+from kcloader.resource.protocol_mapper import ClientProtocolMapperManager
 
 logger = logging.getLogger(__name__)
 
@@ -97,43 +100,24 @@ class SingleClientResource(SingleResource):
         })
         self.datadir = resource['datadir']
 
-        self.client_role_manager = None  # we do not have client id yet...
+        # we do not have client id yet, managers cannot be created
+        self.client_role_manager = None
+        self.client_default_client_scope_manager = None
+        self.client_optional_client_scope_manager = None
+        self.client__scope_mappings__allclients_manager = None
+        self.client__scope_mappings__realm_manager = None
+        self.client__protocol_mapper__manager = None
 
-    def publish_scopes(self):
-        state = True
-        [scopes_path_exist, scopes_path] = lookup_child_resource(self.resource_path, 'scope-mappings.json')
-        if not scopes_path_exist:
-            return state
-        scopes_objects = read_from_json(scopes_path)
-        assert isinstance(scopes_objects, list)
-        if not scopes_objects:
-            # empty list
-            return state
-        assert isinstance(scopes_objects[0], dict)
+    def publish_scope_mappings(self):
+        state_realm_roles = self.client__scope_mappings__realm_manager.publish()
+        state_clients_roles = self.client__scope_mappings__allclients_manager.publish()
+        return any([state_realm_roles, state_clients_roles])
 
-        clients_api = self.resource.resource_api
-        clients = clients_api.all()
+    def publish_protocol_mappers(self):
+        state_protocol_mapper = self.client__protocol_mapper__manager.publish()
+        return state_protocol_mapper
 
-        #  roles_by_id_api.get_child(roles_by_id_api, ci0_default_roles['id'], "composites")
-        this_client = find_in_list(clients, clientId=self.body["clientId"])
-        this_client_scope_mappings_realm_api = clients_api.get_child(clients_api, this_client["id"], "scope-mappings/realm")
-
-        # master_realm = self.keycloak_api.admin()
-        realm_roles_api = self.keycloak_api.build('roles', self.realm_name)
-        realm_roles = realm_roles_api.all()
-
-        # self.keycloak_api.build('clients', self.realm)
-
-        for scopes_object in scopes_objects:
-            role = find_sub_role(self, clients, realm_roles, clients_roles=None, sub_role=scopes_object)
-            if not role:
-                logger.error(f"sub_role {scopes_object} not found")
-            this_client_scope_mappings_realm_api.create([role])
-
-        # TODO remove scope mappings that are assigned, but are not in json file
-        return state
-
-    def publish_self(self):
+    def publish_self(self, *, include_composite=False):
         """
         If client has configured "defaultRoles", then client role with such name will be
         magically created by server. All other role attributes will be wrong.
@@ -160,14 +144,71 @@ class SingleClientResource(SingleResource):
             self.keycloak_api, self.realm_name, self.datadir,
             clientId=self.body["clientId"], client_id=client["id"], client_filepath=self.resource_path,
         )
+        self.client_default_client_scope_manager = ClientDefaultClientScopeManager(
+            self.keycloak_api, self.realm_name, self.datadir,
+            client_id=client["id"], client_filepath=self.resource_path,
+        )
+        self.client_optional_client_scope_manager = ClientOptionalClientScopeManager(
+            self.keycloak_api, self.realm_name, self.datadir,
+            client_id=client["id"], client_filepath=self.resource_path,
+        )
+        self.client__protocol_mapper__manager = ClientProtocolMapperManager(
+            self.keycloak_api, self.realm_name, self.datadir,
+            client_id=client["id"], requested_doc=self.body.get("protocolMappers", []),
+        )
+
+        if include_composite:
+            # Only in second pass all clients are available, and we can create
+            # ClientScopeMappingsRealmManager/ClientScopeMappingsAllClientsManager instances.
+
+            # Here we need to compute requested_doc - we cannot just use part of json file.
+            # TODO - kcfethcer should output directly readable json file(s).
+            logger.debug(f"client {self.resource_path}")
+            client_dirname = os.path.dirname(self.resource_path)
+            scope_mappings_filepath = os.path.join(client_dirname, "scope-mappings.json")
+            scope_mappings_doc = read_from_json(scope_mappings_filepath)
+            requested_doc_realm = [rr["name"] for rr in scope_mappings_doc if rr["clientRole"] is False]
+            requested_doc_allclients = {}
+            for rr in scope_mappings_doc:
+                if rr["clientRole"] is False:
+                    continue
+                if rr["containerName"] not in requested_doc_allclients:
+                    requested_doc_allclients[rr["containerName"]] = []
+                requested_doc_allclients[rr["containerName"]].append(rr["name"])
+            self.client__scope_mappings__realm_manager = ClientScopeMappingsRealmManager(
+                self.keycloak_api, self.realm_name, self.datadir,
+                client_id=client["id"], requested_doc=requested_doc_realm,
+            )
+            self.client__scope_mappings__allclients_manager = ClientScopeMappingsAllClientsManager(
+                self.keycloak_api, self.realm_name, self.datadir,
+                client_id=client["id"], requested_doc=requested_doc_allclients,
+            )
 
         return state
 
     def publish(self, *, include_composite=True):
-        state = self.publish_self()
+        state_self = self.publish_self(include_composite=include_composite)
         state_roles = self.client_role_manager.publish(include_composite=include_composite)
-        state_scopes = self.publish_scopes()
-        return any([state, state_roles, state_scopes])
+        if include_composite:
+            # Other clients might not be yet created in first pass (new server bootstrap).
+            # Setup scope_mappings in second pass.
+            # Assigned role can be removed even if it is assigned to some client scope-mappings -
+            # we do not need two-stage publish.
+            state_scope_mappings = self.publish_scope_mappings()
+        else:
+            state_scope_mappings = False
+        setup_new_links = include_composite
+        state_default_client_scopes = self.client_default_client_scope_manager.publish(setup_new_links=setup_new_links)
+        state_optional_client_scopes = self.client_optional_client_scope_manager.publish(setup_new_links=setup_new_links)
+        state_protocol_mappers = self.publish_protocol_mappers()
+        return any([
+            state_self,
+            state_roles,
+            state_scope_mappings,
+            state_default_client_scopes,
+            state_optional_client_scopes,
+            state_protocol_mappers,
+        ])
 
     def is_equal(self, obj):
         """
@@ -175,15 +216,15 @@ class SingleClientResource(SingleResource):
         :return: True if content in self.body is same as in obj
         """
         # self.body is already sorted
-        obj1 = copy(self.body)
-        obj2 = copy(obj)
+        obj1 = deepcopy(self.body)
+        obj2 = deepcopy(obj)
         for oo in [obj1, obj2]:
             oo.pop("id", None)
-            # authenticationFlowBindingOverrides is not implemented yet, ignore it
+            # TODO authenticationFlowBindingOverrides is not implemented yet, ignore it
             oo["authenticationFlowBindingOverrides"] = {}
-            # sort scopes
-            oo["defaultClientScopes"] = sorted(oo["defaultClientScopes"])
-            oo["optionalClientScopes"] = sorted(oo["optionalClientScopes"])
+            # defaultClientScopes/optionalClientScopes are intentionally not stored by kcfetcher
+            oo.pop("defaultClientScopes", None)
+            oo.pop("optionalClientScopes", None)
             if "protocolMappers" in oo:
                 # remove id from protocolMappers
                 for protocol_mapper in oo["protocolMappers"]:
